import { Grid, Box } from "@material-ui/core";
import React from "react";
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import { KeyValueTable } from "../common/KeyValueTable/KeyValueTable";
import { IOCBadge } from "./IOCBadge";
import AccessControl from "../auth/AccessControl";

export function diff(o1, o2) {
  let diff = Object.keys(o2).reduce((diff, key) => {
    if (o1[key] === o2[key]) return diff
    return {
      ...diff,
      [key]: o2[key]
    }
  }, {})
  return diff;
}

const defaultSubset = ({ name, description, host, active, status, git, version }) => ({ name, description, host, active, status, git, version });

export function IOCDetails({ ioc, getSubset=defaultSubset, alert, buttons }) {
  ioc = { ...ioc };
  const subset = getSubset(ioc);
  
  return (
    <>
      <Grid container>
        {alert && <Grid item xs={12}>
          {alert}
        </Grid>}
        <Grid item xs={12}>
          <Box display="flex" flexDirection="row-reverse" p={2} m={1}>
            <AccessControl 
              allowedRoles={["DeploymentToolAdmin, DeploymentToolIntegrator"]}
              renderNoAccess={() =><></>}>
                {buttons}
            </AccessControl>
          </Box>
        </Grid>
        <Grid item xs={12}>
          <SimpleAccordion defaultExpanded summary={<IOCBadge ioc={ioc} />} >
            <KeyValueTable obj={subset} variant="overline" />
          </SimpleAccordion>
        </Grid>
      </Grid>
    </>
  );
}