import React from 'react';
import PropTypes from 'prop-types';
import { LinearProgress, Paper } from '@material-ui/core';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import 'primeicons/primeicons.css';
import 'primereact/resources/themes/mdc-light-indigo/theme.css';
import 'primereact/resources/primereact.css';
import 'primeflex/primeflex.css';
import './CustomTable.css';

export function CustomTable(props) {
  const {columns, rows, handleRowClick, totalCount, lazyParams, setLazyParams, columnSort, setColumnSort, rowsPerPage, loading=false} = props;
  
  const dynamicColumns = columns.map((col,i) => {
        return <Column key={col.id} field={col.id} header={col.label} style={{width: col.width}} body={col.body} sortable={col.sortable}/>;
    });

  const handleClick = (event) => {
    if(handleRowClick) {
      handleRowClick(event.data.id);
    }
  };

  const onPage = (event) => {
    setLazyParams(event);
  }

  const onSort = (event) => {
    setColumnSort(event);
  }

  const rowClassName = (rowData) => {

    var resultObject = {};

    //if row is clickable -> change cursor by adding 'a new' className
    if (handleRowClick) {
      resultObject.isLink = true;
    }

    if (rowData.iocNotDeployed) {
      resultObject.iocNotDeployed = true;
    }

    if (rowData.inconsistentState) {
      resultObject.inconsistentState = true;
      return resultObject;
    } else if (rowData.discrepancy) {
      resultObject.discrepancy = true;
      return resultObject;
    } else if (rowData.exporterUnknown) {
      resultObject.exporterUnknown = true;
      return resultObject;
    } else if (rowData.deploymentFailed) {
      resultObject.deploymentFailed = true;
    }

    return resultObject;
  }

  return (
    <div className="ccce-custom-table">
      <Paper>
        <DataTable
          className="p-ccce-custom-table"
          lazy
          first={lazyParams.first}
          onPage={onPage}
          onSort={onSort}
          sortField={columnSort.sortField} 
          sortOrder={columnSort.sortOrder}
          value={rows}
          stripedRows
          scrollable 
          resizableColumns 
          columnResizeMode="fit"
          paginator
          paginatorTemplate="CurrentPageReport FirstPageLink PrevPageLink PageLinks NextPageLink LastPageLink RowsPerPageDropdown"
          currentPageReportTemplate="Showing {first} to {last} of {totalRecords}" rows={lazyParams.rows} rowsPerPageOptions={rowsPerPage}
          rowHover
          onRowClick={(event) => handleClick(event)}
          rowClassName={rowClassName}
          totalRecords={totalCount}
          emptyMessage={loading ? <LinearProgress /> : "No records found"}
        >
          {dynamicColumns}
        </DataTable>
      </Paper>
    </div>
  );
}

CustomTable.propTypes = {
  columns: PropTypes.arrayOf(PropTypes.shape({id: PropTypes.string.isRequired})).isRequired,
  rows: PropTypes.array.isRequired,
  handleRowClick: PropTypes.func,
};