import { Box, Button, Container, Grid } from "@material-ui/core";
import { BugReport } from "@material-ui/icons";
import React, { useCallback, useState } from "react";
import { ErrorBoundary, useErrorHandler } from "react-error-boundary";
import { useEffectOnMount } from "../../../hooks/MountEffects";
import { theme } from "../../../style/Theme";

function AwSnap({ error, resetErrorBoundary }) {
  console.log("AwSnap", error);
  return (
    <>
      <Grid container justifyContent="center">
        <BugReport style={{ fontSize: 128, color: theme.palette.primary.main }} />
      </Grid>
      <Grid container justifyContent="center">
        <h1 style={{ textAlign: "center" }}>💥 Whoops! Something went wrong 💥</h1>
      </Grid>
      <Grid container justifyContent="center">
        <Button
          variant="contained"
          color="secondary"
          style={{ backgroundColor: theme.palette.primary.main }}
          onClick={resetErrorBoundary}>
          Start Over
        </Button>
      </Grid>
      <Grid container justifyContent="center">
        <Box p={10} overflow="auto">
          <pre>{error.stack}</pre>
        </Box>
      </Grid >
    </>
  );
}

function WindowErrorRedirect({ children }) {
  const handleError = useErrorHandler();
  const wtf = useCallback((event) => {
    handleError(event.error);
  }, [handleError]);

  useEffectOnMount(() => {
    window.addEventListener('error', wtf);
    return function cleanup() {
      window.removeEventListener('error', wtf);
    }
  })

  return children;
}

export function AppErrorBoundary({ children }) {
  return (
    <ErrorBoundary
      FallbackComponent={AwSnap}
      onReset={() => {
        const reload = window.confirm("The application will reload");
        if (reload) {
          window.location.href = "/";
        }
      }}
    >
      <WindowErrorRedirect>
        {children}
      </WindowErrorRedirect>
    </ErrorBoundary>
  );
}

export function Break() {
  const [broken, setBroken] = useState(false);
  const Bomb = ({ where }) => {
    throw new Error(`KABOOM: You broke the UI ${where}`);
  }
  return (
    <Container>
      <Button
        variant="contained" onClick={() => setBroken(true)}>
        Break the UI during render
      </Button>
      {broken && <Bomb where="during Render" />}
      <Button
        variant="contained" onClick={(event) => {
          Bomb({ where: "in a Callback" });
        }
        }>
        Break the UI asynchronously
      </Button>
    </Container>
  );
}